import epics
import json
import time
import sys
from threading import Thread


class contactsScreen(Thread):
    def __init__(self, stop_signal):
        Thread.__init__(self)
        self.stop_signal = stop_signal

    def run(self):
        pvs = [
            "NSO-LCR:Ops:SID",
            "NSO-LCR:Ops:Msg",
            "NSO-LCR:Ops:Phone",
            "NSO-LCR:Ops:MSL",
            "NSO-LCR:Ops:MSLPhone",
            "NSO-LCR:Ops:MSLEmail",
            "NSO-LCR:Ops:TSMsg",
            "NSO-LCR:Ops:TSPhone",
            "NSO-LCR:Ops:TSSL",
            "NSO-LCR:Ops:TSMobile",
            "NSO-LCR:Ops:TSEmail",
            "NSO-LCR:Ops:OCMsg",
            "NSO-LCR:Ops:OCPhone",
            "NSO-LCR:Ops:OC",
            "NSO-LCR:Ops:OCMobile",
            "NSO-LCR:Ops:OCEmail",
            "NSO-LCR:Ops:OCCMsg",
            "NSO-LCR:Ops:OCCPhone",
            "NSO-LCR:Ops:OCC",
            "NSO-LCR:Ops:OCCMobile",
            "NSO-LCR:Ops:OCCEmail",
            "NSO-LCR:Ops:OCIMsg",
            "NSO-LCR:Ops:OCIPhone",
            "NSO-LCR:Ops:OCI",
            "NSO-LCR:Ops:OCIMobile",
            "NSO-LCR:Ops:OCIEmail",
            "NSO-LCR:Ops:OCPSSMsg",
            "NSO-LCR:Ops:OCPSSPhone",
            "NSO-LCR:Ops:OCPSS",
            "NSO-LCR:Ops:OCPSSMobile",
            "NSO-LCR:Ops:OCPSSEmail",
            "NSO-LCR:Ops:OCEMsg",
            "NSO-LCR:Ops:OCEPhone",
            "NSO-LCR:Ops:OCE",
            "NSO-LCR:Ops:OCEMobile",
            "NSO-LCR:Ops:OCEEmail",
        ]

        epics_dict = {}
        for pv in pvs:
            epics_dict[pv] = epics.PV(pv, auto_monitor=True)

        while not self.stop_signal.isSet():
            try:
                json_dict = {}
                for pv in epics_dict:
                    json_dict[pv] = {}
                    if epics_dict[pv].connected:
                        json_dict[pv]["units"] = epics_dict[
                            pv
                        ].units  # I set the units for all the PVs
                        json_dict[pv]["timestamp"] = epics_dict[
                            pv
                        ].timestamp  # I set the units for all the PVs
                        value = epics_dict[pv].value
                        if type(value) == float:
                            json_dict[pv]["value"] = round(value, 3)
                        else:
                            json_dict[pv]["value"] = value
                    else:
                        json_dict[pv][
                            "units"
                        ] = "n.c."  # If the PV is disconnected I assign n.c. as unit an blank value
                        json_dict[pv][
                            "timestamp"
                        ] = ""  # If the PV is disconnected I assign n.c. as unit an blank value
                        json_dict[pv]["value"] = ""

                tmp_json = json.dumps(json_dict)  # Create the JSON from dictionary
                tmp_json = tmp_json.replace(
                    "NaN", "0"
                )  # Replace the occurences of NaN with 0
                tmp_json = tmp_json.replace(
                    "Infinity", "0"
                )  # Replace the infinities with 0
                with open(
                    "/var/www/data/contacts.json", "w"
                ) as datafile:  # Write the final file
                    datafile.write(tmp_json)
            except Exception:
                print(sys.exc_info())
                time.sleep(1)
            time.sleep(0.5)  # Refresh every half a second

        for (
            pv
        ) in (
            epics_dict
        ):  # When the stop signal is set, it cancels all the auto monitor and disconnects
            epics_dict[pv].clear_auto_monitor()
            epics_dict[pv].disconnect()
