import epics
import json
import time
import sys
from threading import Thread


class odhScreen(Thread):
    def __init__(self, stop_signal):
        Thread.__init__(self)
        self.stop_signal = stop_signal

    def run(self):
        pvs = [
            "HCB-ACH:ODH-O2iM-1:O2Level",
            "HCB-ACH:ODH-O2iM-1:ODHWarning",
            "HCB-ACH:ODH-O2iM-1:ODHAlarm",
            "HCB-ACH:ODH-O2iM-2:O2Level",
            "HCB-ACH:ODH-O2iM-2:ODHWarning",
            "HCB-ACH:ODH-O2iM-2:ODHAlarm",
            "HCB-ACH:ODH-O2iM-3:O2Level",
            "HCB-ACH:ODH-O2iM-3:ODHWarning",
            "HCB-ACH:ODH-O2iM-3:ODHAlarm",
            "HCB-ACH:ODH-O2iM-4:O2Level",
            "HCB-ACH:ODH-O2iM-4:ODHWarning",
            "HCB-ACH:ODH-O2iM-4:ODHAlarm",
            "HCB-ACH:ODH-O2iM-5:O2Level",
            "HCB-ACH:ODH-O2iM-5:ODHWarning",
            "HCB-ACH:ODH-O2iM-5:ODHAlarm",
            "HCB-TCH:ODH-O2iM-6:O2Level",
            "HCB-TCH:ODH-O2iM-6:ODHWarning",
            "HCB-TCH:ODH-O2iM-6:ODHAlarm",
            "HCB-TCH:ODH-O2iM-7:O2Level",
            "HCB-TCH:ODH-O2iM-7:ODHWarning",
            "HCB-TCH:ODH-O2iM-7:ODHAlarm",
            "HCB-TCH:ODH-O2iM-8:O2Level",
            "HCB-TCH:ODH-O2iM-8:ODHWarning",
            "HCB-TCH:ODH-O2iM-8:ODHAlarm",
            "HCB-TCH:ODH-O2iM-9:O2Level",
            "HCB-TCH:ODH-O2iM-9:ODHWarning",
            "HCB-TCH:ODH-O2iM-9:ODHAlarm",
            "HCB-HPGS:ODH-O2iM-10:O2Level",
            "HCB-HPGS:ODH-O2iM-10:ODHWarning",
            "HCB-HPGS:ODH-O2iM-10:ODHAlarm",
            "HCB-HPGS:ODH-O2iM-11:O2Level",
            "HCB-HPGS:ODH-O2iM-11:ODHWarning",
            "HCB-HPGS:ODH-O2iM-11:ODHAlarm",
            "HCB-DogSh:ODH-O2iM-12:O2Level",
            "HCB-DogSh:ODH-O2iM-12:ODHWarning",
            "HCB-DogSh:ODH-O2iM-12:ODHAlarm",
            "CXB-CXH:ODH-O2iM-13:O2Level",
            "CXB-CXH:ODH-O2iM-13:ODHWarning",
            "CXB-CXH:ODH-O2iM-13:ODHAlarm",
            "CXB-CXH:ODH-O2iM-14:O2Level",
            "CXB-CXH:ODH-O2iM-14:ODHWarning",
            "CXB-CXH:ODH-O2iM-14:ODHAlarm",
            "CXB-CXH:ODH-O2iM-15:O2Level",
            "CXB-CXH:ODH-O2iM-15:ODHWarning",
            "CXB-CXH:ODH-O2iM-15:ODHAlarm",
            "CXB-CXH:ODH-O2iM-16:O2Level",
            "CXB-CXH:ODH-O2iM-16:ODHWarning",
            "CXB-CXH:ODH-O2iM-16:ODHAlarm",
            "CXB-CXH:ODH-O2iM-17:O2Level",
            "CXB-CXH:ODH-O2iM-17:ODHWarning",
            "CXB-CXH:ODH-O2iM-17:ODHAlarm",
            "CXB-CXH:ODH-O2iM-18:O2Level",
            "CXB-CXH:ODH-O2iM-18:ODHWarning",
            "CXB-CXH:ODH-O2iM-18:ODHAlarm",
            "CXB-CXH:ODH-O2iM-19:O2Level",
            "CXB-CXH:ODH-O2iM-19:ODHWarning",
            "CXB-CXH:ODH-O2iM-19:ODHAlarm",
            "CTLG-CS:ODH-O2iM-20:O2Level",
            "CTLG-CS:ODH-O2iM-20:ODHWarning",
            "CTLG-CS:ODH-O2iM-20:ODHAlarm",
            "CTLG-CS:ODH-O2iM-21:O2Level",
            "CTLG-CS:ODH-O2iM-21:ODHWarning",
            "CTLG-CS:ODH-O2iM-21:ODHAlarm",
            "CTLG-CS:ODH-O2iM-22:O2Level",
            "CTLG-CS:ODH-O2iM-22:ODHWarning",
            "CTLG-CS:ODH-O2iM-22:ODHAlarm",
            "KG-GTA:ODH-O2iM-1:O2Level",
            "KG-GTA:ODH-O2iM-1:O2Lvl19_5",
            "KG-GTA:ODH-O2iM-1:O2Lvl18",
            "KG-GTA:ODH-O2iM-2:O2Level",
            "KG-GTA:ODH-O2iM-2:O2Lvl19_5",
            "KG-GTA:ODH-O2iM-2:O2Lvl18",
        ]

        epics_dict = {}
        for pv in pvs:
            epics_dict[pv] = epics.PV(pv, auto_monitor=True)

        while not self.stop_signal.isSet():
            try:
                json_dict = {}
                for pv in epics_dict:
                    if "Level" in pv:
                        json_dict[pv] = {}
                        status = False
                        if epics_dict[pv].connected:
                            json_dict[pv]["units"] = epics_dict[pv].units
                            json_dict[pv]["timestamp"] = epics_dict[pv].timestamp
                            json_dict[pv]["status"] = 0
                            value = epics_dict[pv].value
                            if type(value) == float:
                                json_dict[pv]["value"] = round(value, 3)
                            else:
                                json_dict[pv]["value"] = value
                        else:
                            json_dict[pv]["units"] = "n.c."
                            json_dict[pv]["timestamp"] = ""
                            json_dict[pv]["value"] = "Offline"
                            json_dict[pv]["status"] = 0
                            status = False

                        if (
                            pv == "KG-GTA:ODH-O2iM-1:O2Level"
                            or pv == "KG-GTA:ODH-O2iM-2:O2Level"
                        ):
                            if epics_dict[pv[0:-5] + "Lvl19_5"].connected:
                                if epics_dict[pv[0:-5] + "Lvl19_5"].value == 0:
                                    json_dict[pv]["status"] = 1
                                status = True
                            if epics_dict[pv[0:-5] + "Lvl18"].connected:
                                if epics_dict[pv[0:-5] + "Lvl18"].value == 0:
                                    json_dict[pv]["status"] = 2
                                status = True
                            if not status:
                                json_dict[pv]["status"] = 3
                        else:
                            if epics_dict[pv[0:-6] + "DHWarning"].connected:
                                if epics_dict[pv[0:-6] + "DHWarning"].value == 1:
                                    json_dict[pv]["status"] = 1
                                status = True
                            if epics_dict[pv[0:-6] + "DHAlarm"].connected:
                                if epics_dict[pv[0:-6] + "DHAlarm"].value == 1:
                                    json_dict[pv]["status"] = 2
                                status = True
                            if not status:
                                json_dict[pv]["status"] = 3

                tmp_json = json.dumps(json_dict)
                tmp_json = tmp_json.replace("NaN", "0")
                tmp_json = tmp_json.replace("Infinity", "0")
                with open("/var/www/data/odh.json", "w") as datafile:
                    datafile.write(tmp_json)
            except Exception:
                print(sys.exc_info())
                time.sleep(1)
            time.sleep(0.5)

        for pv in epics_dict:
            epics_dict[pv].clear_auto_monitor()
            epics_dict[pv].disconnect()
