import epics
import json
import time
import sys
from threading import Thread


class ts2_tpcircuitsScreen(Thread):
    def __init__(self, stop_signal):
        Thread.__init__(self)
        self.stop_signal = stop_signal

    def run(self):
        pvs = [
            "TS2-010CRM:Cryo-LT-001:MeasValue",
            "TS2-010CRM:Cryo-LT-002:MeasValue",
            "TS2-010CRM:Cryo-TE-010:MeasValue",
            "TS2-010CRM:Cryo-TE-020:MeasValue",
            "TS2-010CRM:Cryo-TE-030:MeasValue",
            "TS2-010CRM:Cryo-TE-007:MeasValue",
            "TS2-010CRM:Cryo-TE-008:MeasValue",
            "TS2-010CRM:Cryo-TE-009:MeasValue",
            "TS2-010CRM:Cryo-TE-002:MeasValue",
            "TS2-010CRM:Cryo-TE-093:MeasValue",
            "TS2-010CRM:Cryo-PT-004:MeasValue",
            "TS2-010CRM:Cryo-PT-003:MeasValue",
            "TS2-010CRM:Cryo-TE-091:MeasValue",
            "TS2-010CRM:Cryo-TE-092:MeasValue",
            "TS2-010CRM:Cryo-TE-094:MeasValue",
            "TS2-010CDL:Cryo-PT-82301:MeasValue",
            "TS2-010CRM:Cryo-PT-002:MeasValue",
            "TS2-010CRM:Cryo-TE-001:MeasValue",
        ]

        epics_dict = {}
        for pv in pvs:
            epics_dict[pv] = epics.PV(pv, auto_monitor=True)

        while not self.stop_signal.isSet():
            try:
                json_dict = {}
                for pv in epics_dict:
                    json_dict[pv] = {}
                    if epics_dict[pv].connected:
                        json_dict[pv]["units"] = epics_dict[pv].units
                        json_dict[pv]["timestamp"] = epics_dict[pv].timestamp
                        value = epics_dict[pv].value
                        if type(value) == float:
                            json_dict[pv]["value"] = round(value, 3)
                        else:
                            json_dict[pv]["value"] = value
                    else:
                        json_dict[pv]["units"] = "n.c."
                        json_dict[pv]["timestamp"] = ""
                        json_dict[pv]["value"] = ""

                tmp_json = json.dumps(json_dict)
                tmp_json = tmp_json.replace("NaN", "0")
                tmp_json = tmp_json.replace("Infinity", "0")
                with open("/var/www/data/ts2_tpcircuits.json", "w") as datafile:
                    datafile.write(tmp_json)
            except Exception:
                print(sys.exc_info())
                time.sleep(1)
            time.sleep(0.5)

        for pv in epics_dict:
            epics_dict[pv].clear_auto_monitor()
            epics_dict[pv].disconnect()
